//
//  PlayerAdaptSwitch.m
//  DebugPlayer
//
//  Created by baidu on 2019/10/12.
//

#import "PlayerAdaptSwitch.h"
#import "Defines.h"
#import <BDCloudMediaAdaptive/BDCloudMediaAdaptive.h>

@interface PlayerAdaptSwitch ()
@property(nonatomic, strong) BDCloudMediaAdaptiveControl *adaptiveControl;
@property(nonatomic, assign) NSInteger inputType;
@property(nonatomic, assign) BOOL isSwitching;
@property(nonatomic, strong) NSTimer *timer;
@end

@implementation PlayerAdaptSwitch

- (instancetype)initWithPlayer:(BDCloudMediaPlayerController *)player
                     inputType:(NSInteger)inputType
                     mediaList:(NSArray *)mediaList
                      adaptive:(BOOL)adaptive {
    if (self = [super init]) {
        _inputType = inputType;
        [self player:player inputType:inputType mediaList:mediaList];
        if (adaptive) {
            [self adaptiveControlWithInputType:inputType mediaList:mediaList];
            [self startTimer:player];
        }
    }
    return self;
}

- (void)player:(BDCloudMediaPlayerController *)player
     inputType:(NSInteger)inputType
     mediaList:(NSArray *)mediaList {
    [player setMediaItemsInputType:inputType];
    if (mediaList.count <= 0) {
        return;
    }
    NSMutableArray<BDCloudMediaPlayerMeidaItem *> *items = [NSMutableArray array];
    for (int i = 0; i < mediaList.count; i++) {
        VideoInput *input = mediaList[i];
        BDCloudMediaPlayerMeidaItem *item = [BDCloudMediaPlayerMeidaItem item:input.url];
        [items addObject:item];
    }
    [player setMediaItems:items];
    // 加速切换效率，可以将缓冲设置小，建议为视频的1-2个视频gop长度
    [player setMaxCacheDurationInSeconds:10];
    [player setOptionIntValue:1
                       forKey:@"smooth_switch_seek_flag"
                   ofCategory:BDCloudMediaPlayerOptionCategoryFormat];
    [player setMediaItemIndex:0];
}

- (void)adaptiveControlWithInputType:(NSInteger)inputType
                           mediaList:(NSArray *)mediaList {
    self.adaptiveControl = [BDCloudMediaAdaptiveControl defaultControl];
    NSMutableArray<BDCloudMediaAdaptiveItem *> *adaptItems = [NSMutableArray array];
    for (int i = 0; i < mediaList.count; i++) {
        VideoInput *input = mediaList[i];
        BDCloudMediaAdaptiveItem *adaptItem = [[BDCloudMediaAdaptiveItem alloc] init:input.url
                                                                             bitrate:input.kbps * 1000];
        [adaptItems addObject:adaptItem];
    }
    [self.adaptiveControl setItems:adaptItems];
    [self.adaptiveControl setMaxCacheDuration:10];
    [self.adaptiveControl setCurrentPlayIndex:0];
}

- (NSInteger)currentPlayIndex:(BDCloudMediaPlayerController *)player {
    return [player mediaItemIndex];
}

- (NSArray *)getSupportedBitrates:(BDCloudMediaPlayerController *)player {
    return [player getMediaItems];
}

- (void)changeBitrate:(NSInteger)index player:(BDCloudMediaPlayerController *)player {
    [player setMediaItemIndex:index];
}

- (void)onPrepared:(BDCloudMediaPlayerController *)player {
    if (self.inputType == BDCloudMediaPlayerMediaItemInputTypeMasterPlistHLS) {
        NSMutableArray<BDCloudMediaAdaptiveItem *> *adaptItems = [NSMutableArray array];
        NSArray *mediaItems = [player getMediaItems];
        for (BDCloudMediaPlayerMeidaItem *item in mediaItems) {
            BDCloudMediaAdaptiveItem *adaptItem = [[BDCloudMediaAdaptiveItem alloc] init:item.contentString
                                                                                 bitrate:item.bitrate];
            [adaptItems addObject:adaptItem];
        }
        [self.adaptiveControl setItems:adaptItems];
    }
}

- (void)onStreamChangeStart:(BDCloudMediaPlayerController *)player {
    _isSwitching = true;
}

- (void)onStreamChagneEnd:(BDCloudMediaPlayerController *)player {
    _isSwitching = false;
    NSInteger mediaIndex = [player mediaItemIndex];
    [self.adaptiveControl setCurrentPlayIndex:mediaIndex];
}

- (void)onTCPLoadSize:(NSInteger)loadSize
               player:(BDCloudMediaPlayerController *)player {
    [self.adaptiveControl addLoadSize:loadSize loadSpeed:player.downloadSpeed * 8];
    [self.adaptiveControl setCacheDuration:player.playableDuration - player.currentPlaybackTime];
}

- (void)changePredict:(BDCloudMediaPlayerController *)player {
    BDCloudMediaAdaptiveItem *item =  [self.adaptiveControl getPredictMediaItem];
    if (!self.isSwitching) {
        [player setMediaItemIndex:item.mediaIndex];
    }
}

- (void)startTimer:(BDCloudMediaPlayerController *)player {
    __weak typeof(self) wSelf = self;
    self.timer = [NSTimer scheduledTimerWithTimeInterval:0.5
                                                 repeats:true
                                                   block:^(NSTimer * _Nonnull timer) {
                                                       __strong typeof(self) sSelf = wSelf;
                                                       [sSelf changePredict:player];
                                                   }];
}

- (void)stopTimer {
    if (self.timer) {
        [self.timer invalidate];
        self.timer = nil;
    }
}

- (void)dealloc {
    [self stopTimer];
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}
@end
