//
//  PlayerSource.m
//  DebugPlayer
//
//  Created by baidu on 2019/10/12.
//

#import "PlayerSource.h"
#import <BDCloudMediaSource/BDCloudMediaSource.h>

@interface PlayerSource ()
@property(nonatomic, weak)BDCloudMediaPlayerController *player;
@end

@implementation PlayerSource

+ (PlayerSource *)defaultModel {
    static PlayerSource *model;
    static dispatch_once_t onceToken;
    dispatch_once(&onceToken, ^{
        model = [[PlayerSource alloc] init];
    });
    return model;
}

- (void)downLoad:(NSString *)url size:(int64_t)size {
    [[BDCloudMediaSourceManager sharedInstance] download:url
                                                    size:size
                                                complete:^(BOOL succ, NSError *error) {
                                                    if (!succ && error) {
                                                        NSLog(@"media source download fail = %@", error);
                                                    }
                                                }];
}

- (void)enablePlayer:(BDCloudMediaPlayerController *)player {
    _player = player;
    player.proxy = [BDCloudMediaSourceManager sharedInstance];
    [self addObserver];
}

- (void)addObserver {
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onMediaSourcePlayOpen:)
                                                 name:BDCloudMediaSourcePlayOpenNotification
                                               object:nil];
}

- (void)removeObserver {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)onMediaSourcePlayOpen:(NSNotification *)notification {
    if (notification.object != self.player) {
        return;
    }
    NSLog(@"media source play open");
}

- (void)dealloc {
    [self removeObserver];
}

@end
