//
//  PlayerSpriteModel.m
//  DebugPlayer
//
//  Created by baidu on 2019/10/12.
//

#import "PlayerSpriteModel.h"
#import <BDCloudMediaSprite/BDCloudMediaSprite.h>

@interface PlayerSpriteModel ()
@property(nonatomic, strong)BDCloudSpriteManager *spriteManager;
@end

@implementation PlayerSpriteModel

- (instancetype)initRow:(NSInteger)row
                 column:(NSInteger)column
               duration:(NSTimeInterval)duration
              imageList:(NSArray *)imageList {
    if (self = [super init]) {
        BDCloudSpriteRenderConfiguration *render = [BDCloudSpriteRenderConfiguration defaultConfiguration];
        render.row = row;
        render.column = column;
        render.thumbnailDuration = duration;
        BDCloudSpriteDownloadConfiguration *download = [BDCloudSpriteDownloadConfiguration defaultConfiguration];
        download.imageList = imageList;
        download.maxThreadNum = 5;
        self.spriteManager = [[BDCloudSpriteManager alloc] initWithRenderConfig:render downloadConfig:download];
        self.spriteManager.renderView.hidden = true;
    }
    return self;
}

- (void)startSpriteDownload:(void(^)(BOOL succ))callBack {
    [self.spriteManager startDownload:^void(BOOL succ, NSError *error) {
        if (callBack) {
            callBack(succ);
        }
    }];
}

- (void)stopSpriteDownload {
    [self.spriteManager stopDownload];
}

- (void)updateSpriteRenderSize:(UIView *)superView {
    if (self.spriteManager.renderView.superview != superView) {
        [self.spriteManager.renderView removeFromSuperview];
        [superView addSubview:self.spriteManager.renderView];
    }
    self.spriteManager.renderView.frame = CGRectMake(0, 0, self.spriteManager.renderSize.width,
                                                     self.spriteManager.renderSize.height);
    self.spriteManager.renderView.center = CGPointMake(superView.bounds.size.width / 2,
                                                       superView.bounds.size.height / 2);
}

- (void)showSpriteRender:(NSTimeInterval)position {
    if (self.spriteManager.status != BDCloudSpriteStatusSuccess) {
        return;
    }
    self.spriteManager.renderView.hidden = false;
    [self.spriteManager renderTime:position];
}

- (void)hideSpriteRender {
    self.spriteManager.renderView.hidden = true;
}
@end
