//
//  PlayerViewModel.m
//  VideoPlayer
//
//  Created by 白璐 on 16/9/20.
//  Copyright © 2016年 baidu. All rights reserved.
//

#import "PlayerViewModel.h"
#import "Defines.h"
#import "PlayerModel.h"
#import "PlayerSpriteModel.h"
#import "PlayerVRModel.h"
#import "PlayerAdaptSwitch.h"
#import "PlayerSource.h"
@interface PlayerViewModel ()
@property(nonatomic, strong) PlayerModel* model;
@property(nonatomic, strong) BDCloudMediaPlayerController* player;
@property(nonatomic, strong) PlayerSpriteModel *sprite;
@property(nonatomic, strong) PlayerVRModel *vrModel;
@property(nonatomic, strong) PlayerAdaptSwitch *adaptSwitch;
@property(nonatomic, strong) NSDate *date;
@end

@implementation PlayerViewModel

- (instancetype)initWithURL:(NSString*)url
                      title:(NSString*)title
               downloadable:(BOOL)downloadable
                      token:(NSString *)token {
    if (self = [super init]) {
        _model = [[PlayerModel alloc] initWithURL:url
                                            title:title
                                     downloadable:downloadable
                                            token:token];
        _date = [NSDate date];
        [self setupPlayer];
        [self setupNotifications];
    }
    
    return self;
}

- (NSString *)diffDate:(NSDate *)date {
    return [NSString stringWithFormat:@"%.2f",(([NSDate date].timeIntervalSince1970 - date.timeIntervalSince1970) * 1000)];
}

- (void)setSprite:(NSInteger)row
           column:(NSInteger)column
         duration:(NSTimeInterval)duration
        imageList:(NSArray *)imageList {
    self.sprite = [[PlayerSpriteModel alloc] initRow:row
                                              column:column
                                            duration:duration
                                           imageList:imageList];
    __weak typeof(self) wSelf = self;
    [self.sprite startSpriteDownload:^(BOOL succ) {
        __strong typeof(self) sSelf = wSelf;
        if (succ) {
            [sSelf.sprite updateSpriteRenderSize:sSelf.player.view];
        }
    }];
}

- (void)setIODemoEnable {
    [[PlayerSource defaultModel] enablePlayer:self.player];
}

- (void)adaptiveWithInputType:(NSInteger)inputType
                    InputList:(NSArray *)inputList
                     adaptive:(BOOL)adaptive{
    self.adaptSwitch = [[PlayerAdaptSwitch alloc] initWithPlayer:self.player
                                                       inputType:inputType
                                                       mediaList:inputList
                                                        adaptive:adaptive];
    if (!adaptive) {
        return;
    }
    __weak typeof(self) wSelf = self;
    self.player.tcpLoadSizeBlock = ^(int64_t loadSize) {
        __strong typeof(self) sSelf = wSelf;
        [sSelf.adaptSwitch onTCPLoadSize:loadSize
                                  player:sSelf.player];
    };
}

- (void)enableVRWithProjectionMode:(NSInteger)projectionMode
                       displayMode:(NSInteger)displayMode
                   interactiveMode:(NSInteger)interactiveMode
                      pinchEnabled:(BOOL)pinchEnabled {
    self.vrModel = [[PlayerVRModel alloc] initWithProjectionMode:projectionMode
                                                     displayMode:displayMode
                                                 interactiveMode:interactiveMode
                                                    pinchEnabled:pinchEnabled];
}

- (void)dealloc {
    [[NSNotificationCenter defaultCenter] removeObserver:self];
}

- (void)layout:(UIView*)parent {
    if (![parent.subviews containsObject:self.player.view]) {
        [parent insertSubview:self.player.view atIndex:0];
    }
    self.player.view.frame = parent.bounds;
    [self.sprite updateSpriteRenderSize:self.player.view];
}

- (void)start {
    if (_model.token && _model.token.length != 0) {
        [self.player setToken:_model.token];
    }
    
//    NSString *document = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, true).lastObject;
//    NSString *logName = [NSString stringWithFormat:@"%f",[[NSDate date] timeIntervalSince1970]];
//    NSString *videoLog = [document stringByAppendingPathComponent:[NSString stringWithFormat:@"%@log.txt",logName]];
//    NSLog(@"videoLog = %@",videoLog);
//    [BDCloudMediaPlayerController setLogPath:videoLog];
//    [BDCloudMediaPlayerController setLogLevel:BDCloudMediaPlayerLogLevelError];
//    [BDCloudMediaPlayerController setLogReport:true];
    
    self.player.shouldAutoplay = YES;
    [self.player prepareToPlay];
    [self.sprite updateSpriteRenderSize:self.player.view];

    [self.actions updateTitle:self.model.title];
    [self.actions startLoadingAnimation];
    
    BOOL hasPrevious = [self.videoSource previousVideo:self.model.url] != nil;
    BOOL hasNext = [self.videoSource nextVideo:self.model.url] != nil;
    [self.actions updatePreviousState:hasPrevious];
    [self.actions updateNextState:hasNext];
    [self.actions updateDownoadable:self.model.downloadable];
    
    [self.actions appendVideoLog:[NSString stringWithFormat:@"prepareToPlay = %@ms \n",[self diffDate:self.date]]];
}

#pragma mark - private methods

- (void)setupPlayer {
    self.player = [[BDCloudMediaPlayerController alloc] initWithContentString:self.model.url];
}

- (void)setupNotifications {
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerPrepared:)
                                                 name:BDCloudMediaPlayerPlaybackIsPreparedToPlayNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerFinish:)
                                                 name:BDCloudMediaPlayerPlaybackDidFinishNotification
                                               object:nil];
    
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerState:)
                                                 name:BDCloudMediaPlayerPlaybackStateDidChangeNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerBufferingStart:)
                                                 name:BDCloudMediaPlayerBufferingStartNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerBufferingUpdate:)
                                                 name:BDCloudMediaPlayerBufferingUpdateNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerBufferingEnd:)
                                                 name:BDCloudMediaPlayerBufferingEndNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerSeekCompleted:)
                                                 name:BDCloudMediaPlayerSeekCompleteNotification
                                               object:nil];
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerNatureSize:)
                                                 name:BDCloudMediaPlayerNaturalSizeChangedNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerStreamChangeStart:)
                                                 name:BDCloudMediaPlayerMediaChangeStartNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerStreamChangeEnd:)
                                                 name:BDCloudMediaPlayerMediaChangeEndNotification
                                               object:nil];
    
    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerFirstFrameRendered:)
                                                 name:BDCloudMediaPlayerFirstVideoFrameRenderedNotification object:nil];

    [[NSNotificationCenter defaultCenter] addObserver:self
                                             selector:@selector(onPlayerVideoDecoderOpen:)
                                                 name:BDCloudMediaPlayerVideoDecoderOpenNotification object:nil];
}

- (void)onPlayerStreamChangeStart:(NSNotification*)notification{
    if (notification.object != self.player) {
        return;
    }
    [self.adaptSwitch onStreamChangeStart:self.player];
    [self.actions showStartChangeMedia];
}

- (void)onPlayerStreamChangeEnd:(NSNotification*)notification{
    if (notification.object != self.player) {
        return;
    }
    NSNumber* result = notification.userInfo[BDCloudMediaPlayerMediaChangeEndResultUserInfoKey];
    switch ([result integerValue]) {
        case BDCloudMediaPlayerMediaChangeEndResultError:
            [self.actions showEndChangeMedia:false];
            break;
        case BDCloudMediaPlayerMediaChangeEndResultSuccess:
            [self.actions showEndChangeMedia:true];
            break;
        default:
            break;
    }
    [self.adaptSwitch onStreamChagneEnd:self.player];
}

- (void)onPlayerFirstFrameRendered:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    [self.actions appendVideoLog:[NSString stringWithFormat:@"onPlayerFirstFrameRendered = %@ms \n",[self diffDate:self.date]]];
}

- (void)onPlayerNatureSize:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    [self.actions updateResolution:self.player.naturalSize];
    [self.actions appendVideoLog:[NSString stringWithFormat:@"onPlayerNatureSize = %@ms \n",[self diffDate:self.date]]];
}

- (void)onPlayerPrepared:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    [self.actions stopLoadingAnimation];
    [self.actions updateDuration:self.player.duration];
    [self.actions updateResolution:self.player.naturalSize];
    
    [self.adaptSwitch onPrepared:self.player];
    NSArray* bitrates = [self.adaptSwitch getSupportedBitrates:self.player];
    NSInteger index = [self.adaptSwitch currentPlayIndex:self.player];
    [self.actions updateBitrateList:bitrates index:index];
    [self.actions appendVideoLog:[NSString stringWithFormat:@"onPlayerPrepared = %@ms \n",[self diffDate:self.date]]];
}

- (void)onPlayerFinish:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    
    [BDCloudMediaPlayerController setLogReport:false];
    NSNumber* reasonNumber = notification.userInfo[BDCloudMediaPlayerPlaybackDidFinishReasonUserInfoKey];
    BDCloudMediaPlayerFinishReason reason = (BDCloudMediaPlayerFinishReason)reasonNumber.integerValue;
    switch (reason) {
        case BDCloudMediaPlayerFinishReasonEnd:
            NSLog(@"player finish with reason: play to end time");
            break;
        case BDCloudMediaPlayerFinishReasonError: {
            NSString *playerLog = [NSString stringWithContentsOfFile:[BDCloudMediaPlayerController logPath]
                                                            encoding:NSUTF8StringEncoding
                                                               error:nil];
            NSLog(@"player finished with reason: error, player this play all log : %@",playerLog);
            break;
        }
        case BDCloudMediaPlayerFinishReasonUser:
            NSLog(@"player finished with reason: stopped by user");
            break;
        default:
            break;
    }
    
    [BDCloudMediaPlayerController clearDiskLog];
    [self.actions popPlayer];
    [self.actions appendVideoLog:[NSString stringWithFormat:@"onPlayerFinish = %@ms \n",[self diffDate:self.date]]];
}

- (void)onPlayerState:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    
    BDCloudMediaPlayerPlaybackState state = self.player.playbackState;
    [self.actions updatePlayerState:state];
}

- (void)onPlayerBufferingStart:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    [self.actions startLoadingAnimation];
    [self.actions appendVideoLog:[NSString stringWithFormat:@"onPlayerBufferingStart = %@ms \n",[self diffDate:self.date]]];
    NSLog(@"buffering start");
}

- (void)onPlayerBufferingUpdate:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    [self.actions appendVideoLog:[NSString stringWithFormat:@"onPlayerBufferingUpdate = %@ms \n",[self diffDate:self.date]]];
    NSLog(@"buffering update progress %@", notification.userInfo[BDCloudMediaPlayerBufferingProgressUserInfoKey]);
}

- (void)onPlayerBufferingEnd:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    [self.actions stopLoadingAnimation];
    [self.actions appendVideoLog:[NSString stringWithFormat:@"onPlayerBufferingEnd = %@ms \n",[self diffDate:self.date]]];
}

- (void)onPlayerSeekCompleted:(NSNotification*)notification {
    if (notification.object != self.player) {
        return;
    }
    [self.sprite hideSpriteRender];
    NSLog(@"player seek completed");
}

- (void)onPlayerVideoDecoderOpen:(NSNotification *)notification {
    if (notification.object != _player) {
        return;
    }
    [self.vrModel startOnRenderView:_player.view
                     renderPorvider:_player.view
              viaHardwareAccelerate:[_player viaHardwareAccelerate]];
}

- (void)switchVideo:(VideoItem*)item {
    self.model.url = item.url;
    self.model.title = item.title;
    self.model.downloadable = [self.videoSource downloadable:item.url];
    [self.player stop];
    [self.player reset];
    self.player.contentString = item.url;
    [self start];
}

#pragma mark - PlayerControlDelegate

- (BOOL)hasPrevious {
    VideoItem* item = [self.videoSource previousVideo:self.model.url];
    return item != nil;
}

- (BOOL)hasNext {
    VideoItem* item = [self.videoSource nextVideo:self.model.url];
    return item != nil;
}

- (void)play {
    if (self.player.playbackState == BDCloudMediaPlayerPlaybackStatePlaying) {
        [self.player pause];
        [self.vrModel pause];
    } else {
        [self.player play];
        [self.vrModel play];
    }
}

- (void)playPrevious {
    VideoItem* item = [self.videoSource previousVideo:self.model.url];
    [self switchVideo:item];
}

- (void)playNext {
    VideoItem* item = [self.videoSource nextVideo:self.model.url];
    [self switchVideo:item];
}

- (void)download:(void(^)(BOOL cancel))cancelBlock {
    [self.videoSource downloadVideo:self.model.url cancelBlock:cancelBlock];
}

- (void)seek:(NSTimeInterval)position {
    [self.player seek:position];
}

- (void)render:(NSTimeInterval)position {
    [self.sprite showSpriteRender:position];
}

- (void)realtimeVariable:(NSTimeInterval*)playableDuration  position:(NSTimeInterval*)position speed:(double*)speed {
    *playableDuration = self.player.playableDuration;
    *position = self.player.currentPlaybackTime;
    *speed = self.player.downloadSpeed;
}

- (void)scale:(NSInteger)mode {
    if (mode == 0) {
        self.player.scalingMode = BDCloudMediaPlayerScalingModeAspectFit;
    } else {
        self.player.scalingMode = BDCloudMediaPlayerScalingModeAspectFill;
    }
}

- (void)changeBitrate:(NSInteger)index {
    [self.adaptSwitch changeBitrate:index player:self.player];
    [self.actions appendVideoLog:[NSString stringWithFormat:@"changeBitrate start = %@ms \n",[self diffDate:self.date]]];
}

- (UIImage*)snapshot {
    return [self.player thumbnailImageAtCurrentTime];
}

- (void)controlStop {
    [self.player stop];
    [self.sprite stopSpriteDownload];
}
@end
