//
//  VideoListModel.m
//  VideoPlayer
//
//  Created by 白璐 on 16/9/18.
//  Copyright © 2016年 baidu. All rights reserved.
//

#import "VideoListModel.h"

@interface  VideoListModel ()
@end

@implementation VideoListModel

- (instancetype)init {
    if (self = [super init]) {
        _videoList = [NSMutableArray array];
    }
    return self;
}

- (void)loadResource {
    NSString* parentPath = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES)[0];
    NSString* filename = [parentPath stringByAppendingPathComponent:kDefaultUserFilename];
    
    NSFileManager* fm = [NSFileManager defaultManager];
    NSArray* videoList;
    if (![fm fileExistsAtPath:filename]) {
        filename = [[NSBundle mainBundle] pathForResource:kDefaultVideoFilename ofType:nil];
    }
    
    videoList = [NSArray arrayWithContentsOfFile:filename];
    
    for (NSDictionary* dict in videoList) {
        VideoItem* item = [[VideoItem alloc] init];
        item.img = [dict objectForKey:@"img"];
        item.title = [dict objectForKey:@"title"];
        item.detail = [dict objectForKey:@"detail"];
        item.url = [dict objectForKey:@"url"];
        item.token = [dict objectForKey:@"token"];
        
        NSDictionary *mutiBitrateDict = [dict objectForKey:@"mutiBitrate"];
        if (mutiBitrateDict) {
            VideoMutiBitrate *mutiBitrate = [[VideoMutiBitrate alloc] init];
            mutiBitrate.adaptive = [[mutiBitrateDict objectForKey:@"adaptive"] boolValue];
            mutiBitrate.inputType = [[mutiBitrateDict objectForKey:@"inputtype"] integerValue];
            mutiBitrate.inputList = [NSMutableArray array];
            NSArray *inputs = mutiBitrateDict[@"inputList"];
            for (NSDictionary *input in inputs) {
                VideoInput * vinput = [[VideoInput alloc] init];
                [vinput setValuesForKeysWithDictionary:input];
                [mutiBitrate.inputList addObject:vinput];
            }
            item.mutiBitrate = mutiBitrate;
        }
        NSDictionary * spriteDict = [dict objectForKey:@"sprite"];
        if (spriteDict) {
            VideoSprite * sprite = [[VideoSprite alloc] init];
            sprite.row = [[spriteDict objectForKey:@"row"] integerValue];
            sprite.column = [[spriteDict objectForKey:@"column"] integerValue];
            sprite.thumbnailDuration = [[spriteDict objectForKey:@"thumbnailDuration"] doubleValue];
            sprite.imageList = [spriteDict objectForKey:@"imageList"];
            item.sprite = sprite;
        }
        NSDictionary *ioDelegate = [dict objectForKey:@"ioDelegate"];
        if (ioDelegate) {
            VideoIO *io = [[VideoIO alloc] init];
            io.demoEnable = [[ioDelegate objectForKey:@"demoEnable"] boolValue];
            item.io = io;
        }
        NSDictionary *VREnable = [dict objectForKey:@"VREnable"];
        if (VREnable) {
            VRInfo *info = [[VRInfo alloc] init];
            info.projectionMode = [VREnable[@"projectionMode"] integerValue];
            info.displayMode = [VREnable[@"displayMode"] integerValue];
            info.interactiveMode = [VREnable[@"interactiveMode"] integerValue];
            info.pinchEnabled = [VREnable[@"pinchEnabled"] boolValue];
            item.vrinfo = info;
        }
        [self.videoList addObject:item];
    }
}

- (void)saveResource {
    NSMutableArray* array = [NSMutableArray array];
    for (VideoItem* item in self.videoList) {
        NSMutableDictionary* dict = [NSMutableDictionary dictionary];
        [dict setObject:item.img forKey:@"img"];
        [dict setObject:item.title forKey:@"title"];
        [dict setObject:item.detail forKey:@"detail"];
        [dict setObject:item.url forKey:@"url"];
        
        if (item.mutiBitrate) {
            NSMutableDictionary* mutiBitrateDict = [NSMutableDictionary dictionary];
            [mutiBitrateDict setObject:@(item.mutiBitrate.inputType) forKey:@"inputtype"];
            [mutiBitrateDict setObject:@(item.mutiBitrate.adaptive) forKey:@"adaptive"];
            if (item.mutiBitrate.inputList) {
                NSMutableArray *inputs = [NSMutableArray array];
                for (VideoInput *input in item.mutiBitrate.inputList) {
                    NSDictionary *inputDict = @{
                                                @"url":input.url,
                                                @"kbps":@(input.kbps)
                                                };
                    [inputs addObject:inputDict];
                }
                [mutiBitrateDict setObject:inputs forKey:@"inputList"];
            }
            [dict setObject:mutiBitrateDict forKey:@"mutiBitrate"];
        }
        
        if (item.token.length > 0) {
            [dict setObject:item.token forKey:@"token"];
        }
        if (item.sprite) {
            NSMutableDictionary* spriteDict = [NSMutableDictionary dictionary];
            [spriteDict setObject:@(item.sprite.row) forKey:@"row"];
            [spriteDict setObject:@(item.sprite.column) forKey:@"column"];
            [spriteDict setObject:@(item.sprite.thumbnailDuration) forKey:@"thumbnailDuration"];
            [spriteDict setObject:item.sprite.imageList forKey:@"imageList"];
            [dict setObject:spriteDict forKey:@"token"];
        }
        if (item.io) {
            NSMutableDictionary *ioDict = [NSMutableDictionary dictionary];
            [ioDict setObject:@(item.io.demoEnable) forKey:@"demoEnable"];
            [dict setObject:ioDict forKey:@"ioDelegate"];
        }
        if (item.vrinfo) {
            NSMutableDictionary* vrinfo = [NSMutableDictionary dictionary];
            [vrinfo setObject:@(item.vrinfo.projectionMode) forKey:@"projectionMode"];
            [vrinfo setObject:@(item.vrinfo.displayMode) forKey:@"displayMode"];
            [vrinfo setObject:@(item.vrinfo.interactiveMode) forKey:@"interactiveMode"];
            [vrinfo setObject:@(item.vrinfo.pinchEnabled) forKey:@"pinchEnabled"];
            [dict setObject:vrinfo forKey:@"VREnable"];
        }
        [array addObject:dict];
    }
    
    NSString* parentPath = NSSearchPathForDirectoriesInDomains(NSDocumentDirectory, NSUserDomainMask, YES)[0];
    NSString* filename = [parentPath stringByAppendingPathComponent:kDefaultUserFilename];
    [array writeToFile:filename atomically:YES];
}

@end
