//
//  BDCloudMediaProtocol.h
//  BDCloudMediaPlayer
//
//  Created by Fu,Sheng on 2019/4/3.
//  Copyright © 2019年 Baidu. All rights reserved.
//

#import <Foundation/Foundation.h>

@class BDCloudMediaPlayerController;
@protocol BDCloudMediaPlayerProtocol <NSObject>

/**
 是否支持自拉流网络代理。

 @param player 播放器对象示例。
 @param url 资源地址。
 @return 0：支持；-1：不支持。
 */
- (int)onEnableIODelegate:(BDCloudMediaPlayerController *)player url:(NSString *)url;

/**
 打开视频链接网络。

 @param player 播放器对象示例。
 @param url 视频链接URL。
 @return 0：打开成功； -1：打开失败。
 */
- (int)onOpenPlayer:(BDCloudMediaPlayerController *)player url:(NSString *)url;

/**
 网络视频读取。

 @param player 播放器对象示例。
 @param url 资源地址。
 @param buffer 视频数据读取：将自己从网络下载好的数据拷贝到buffer上。
 @param size 需要读取的大小，单位byte。
 @return 本次读取的真实数据大小。
 */
- (int)onReadplayer:(BDCloudMediaPlayerController *)player url:(NSString *)url buffer:(uint8_t *)buffer size:(int)size;

/**
 网络视频定位。

 @param player 播放器对象示例。
 @param url 资源地址。
 @param offset 定位的偏移量，单位byte。
 @param whence 定位方式：SEEK_SET，SEEK_CUR，SEEK_END。
 @return 0：定位成功；-1：定位失败。
 */
- (int)onSeekplayer:(BDCloudMediaPlayerController *)player url:(NSString *)url offset:(int64_t)offset whence:(int)whence;

/**
 关闭视频网络链接。

 @param player 播放器对象示例。
 @param url 资源地址。
 @return 0：关闭成功；1：关闭失败。
 */
- (int)onCloseplayer:(BDCloudMediaPlayerController *)player url:(NSString *)url;

/**
 视频资源总大小

 @param player 播放器对象示例。
 @param url 资源地址。
 @return 视频资源总大小，单位byte。
 */
- (int64_t)onTotalSizePlayer:(BDCloudMediaPlayerController *)player url:(NSString *)url;
@end

