//
//  BDCloudMediaAdaptiveControl.h
//  BDCloudMediaAdaptive
//
//  Created by baidu on 2019/10/11.
//  Copyright © 2019 fusheng. All rights reserved.
//

#import <Foundation/Foundation.h>
#import "BDCloudMediaAdaptiveItem.h"

@interface BDCloudMediaAdaptiveControl : NSObject

/**
 Initialize with default settings

 @return instance.
 */
+ (BDCloudMediaAdaptiveControl *)defaultControl;

/**
 Update adaptive items.

 @param items adaptive items.
 */
- (void)setItems:(NSArray<BDCloudMediaAdaptiveItem *> *)items;

/**
 Update max cache duration.
 Usually is set to player max cache duarion, should be set before player prepare to play.

 @param duration max cache duration in second.
 */
- (void)setMaxCacheDuration:(NSTimeInterval)duration;

/**
 Update current cache duration.
 Usually is set to player current cache duarion, should be set when player current cache changed.

 @param duration current cache duration in second.
 */
- (void)setCacheDuration:(NSTimeInterval)duration;

/**
 Update current media in list.
 Usually used when player changed play media.

 @param index player current play index.
 */
- (void)setCurrentPlayIndex:(NSInteger)index;

/**
 Use current download speed to predict bandwidth.

 @param loadSize download size in byte.
 @param loadSpeed download speed in bps.
 */
- (void)addLoadSize:(NSInteger)loadSize loadSpeed:(int64_t)loadSpeed;

/**
 Get predict bandwidth.

 @return bandwidth in bps.
 */
- (int64_t)getPredictBandwidth;

/**
 Predict a best item for player to play.

 @return adaptive item
 */
- (BDCloudMediaAdaptiveItem *)getPredictMediaItem;

/**
 Update min download size.
 Used to limit predict bandwidth when download size less then min download size.

 @param loadSize min download size in byte. default is 1024 * 0.5.
 */
- (void)setMinLoadSizeInAddLoadSpeed:(int64_t)loadSize;

/**
 Update min download duration.
 Used to limit predict media when download duration less then min download duration.

 @param duration min download duration in second. default is 5s.
 */
- (void)setMinLoadDurationInPredictMediaItem:(NSTimeInterval)duration;
@end
