//
//  BDCloudMediaSourceManager.h
//  BDCloudMediaSource
//
//  Created by Fu,Sheng on 2019/4/25.
//  Copyright © 2019年 Fu,Sheng. All rights reserved.
//

#import <Foundation/Foundation.h>

#if __has_include(<BDCloudMediaPlayer/BDCloudMediaPlayer.h>)
#import <BDCloudMediaPlayer/BDCloudMediaPlayer.h>
#else
#import <BDCloudAVMediaPlayer/BDCloudAVMediaPlayer.h>
#endif

typedef void(^BDCloudMediaDownloadComplete)(BOOL succ, NSError *error);

@interface BDCloudMediaSourceManager : NSObject

/**
 预下载通道数。默认50。
 根据调动预下载，会自动计算并删除之前的加载记录。
 */
@property(nonatomic, assign) NSInteger countLimit;

/**
 最大存储空间。默认500M，单位byte。
 当沙盒磁盘空间不足时，将不在存储。
 */
@property(nonatomic, assign) int64_t sizeLimit;

/**
 下载配置。默认default。
 */
@property(nonatomic, assign) NSURLSessionConfiguration *configuration;

/**
 开启边播边存。默认YES。
 */
@property(nonatomic, assign) BOOL autoWriteCache;

/**
 预下载失败是否继续走网络库。默认NO。
 当预下载失败时，默认使用自拉流服务。设置为NO后将使用的播放器拉流服务。
 */
@property(nonatomic, assign) BOOL enableWhenPreDwonloadFail;

/**
 单次请求超时时间。单位ms，范围(0,1000],默认500ms。
 当弱网时会在播放器超时时间内进行多次请求，即代理方法：（- onReadplayer:url:buffer:size:）直到请求到数据或者超时。
 此接口用于多次请求中的单次请求超时时间。
*/
@property(nonatomic, assign) NSInteger readTimeout;

/**
 初始化。

 @return BDCloudMediaSourceManager 示例
 */
+ (instancetype) sharedInstance;
@end


@interface BDCloudMediaSourceManager (PlayerProtocol)<BDCloudMediaPlayerProtocol>

/**
 设置当前视频的总时长，设置后可加快起播速度。

 @param url 当前视频URL。
 @param dutation 当前视频时长，单位秒。
 */
- (void)setVideoDuration:(NSString *)url duration:(long)dutation ;

/**
 数据缓冲时间。默认10S。
 缓冲时间设置越大，播放越流畅。

 @param url 当前视频URL。
 @param time 缓冲时长，单位秒。
 */
- (void)setVideoCacheTime:(NSString *)url time:(int)time;
@end


@interface BDCloudMediaSourceManager (Download)

/**
 预下载视频一小段，可提升起播速度。

 @param url 当前视频URL。
 @param size 下载大小，单位byte。
 @param complete 下载回调。
 */
- (void)download:(NSString *)url size:(int64_t)size complete:(BDCloudMediaDownloadComplete)complete;

/**
 取消预下载

 @param url 当前视频URL。
 */
- (void)downloadCancel:(NSString *)url;

/**
 根据ULR获取视频是否预下载成功。

 @param url 当前视频URL。
 @return 是否成功。
 */
- (BOOL)downloadInCache:(NSString *)url;

/**
 删除磁盘中下载的资源。

 @param urls 需要删除的URL数组。
 */
- (void)downloadCacheDelete:(NSArray<NSString *> *)urls;

/**
 删除磁盘中全部下载资源。
 */
- (void)downloadCacheDeleteAll;
@end

// 开始使用网络库加载视频。
BDCLOUD_MP_EXTERN NSString* const BDCloudMediaSourcePlayOpenNotification;
