//
//  BDCloudSpriteManager.h
//  BDCloudMediaSprite
//
//  Created by Fu,Sheng on 2019/3/18.
//  Copyright © 2019年 Fu,Sheng. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <UIKit/UIKit.h>
#import "BDCloudSpriteEnum.h"

@class BDCloudSpriteRenderConfiguration;
@class BDCloudSpriteDownloadConfiguration;
@interface BDCloudSpriteManager : NSObject

/**
 雪碧图下载状态。
 */
@property (nonatomic, assign, readonly) BDCloudSpriteStatus status;

/**
 渲染展示View。
 缩率图将在此view上渲染。
 */
@property (nonatomic, strong, readonly) UIView *renderView;

/**
 渲染缩率图分辨率。
 雪碧图下载成功后可获取。
 */
@property (nonatomic, assign, readonly) CGSize renderSize;

/**
 初始化方法。

 @param renderConfig 雪碧图渲染配置。
 @param downloadConfig 雪碧图下载配置。
 @return 雪碧图管理实例。
 */
- (instancetype)initWithRenderConfig:(BDCloudSpriteRenderConfiguration *)renderConfig
                      downloadConfig:(BDCloudSpriteDownloadConfiguration *)downloadConfig;

/**
 开始下载所有雪碧图。

 @param complete 下载完成回调.
 */
- (void)startDownload:(void(^)(BOOL succ, NSError *error))complete;

/**
 取消下载所有雪碧图。
 雪碧图下载状态status为BDCloudSpriteStatusDownloading可取消。
 
 */
- (void)stopDownload;

/**
 清空雪碧图缓存。
 1. 清空缓存后，再次使用雪碧图需要重新下载。
 2. 雪碧图下载状态status为BDCloudSpriteStatusDownloading时不可调用。
 
 */
- (void)clearCache;

/**
 渲染缩率图

 @param time 缩率图时间。
 */
- (void)renderTime:(NSTimeInterval)time;
@end


