//
//  BDCloudVRConfiguration.h
//  BDCloudVRRender
//
//  Created by baidu on 2019/9/9.
//  Copyright © 2019 fusheng. All rights reserved.
//

#import <Foundation/Foundation.h>
#import <AVFoundation/AVFoundation.h>
#import <UIKit/UIKit.h>
#import "BDCloudVRRenderDefines.h"

@interface BDCloudVRConfiguration : NSObject
/**
 VR渲染默认配置实例化方法。

 @return VR渲染配置实例。
 */
+ (BDCloudVRConfiguration *)defaultConfig;

/**
 配置VR渲染图像输入源为AVPlayerItem。

 @param playerItem VR图像输入源。
 */
- (void)setProviderAVPlayerItem:(AVPlayerItem *)playerItem;

/**
 配置VR图像输入源为UIimage图像。
 */
- (void)setProviderImage;

/**
 配置VR图像输入源为百度云播放器渲染视图。

 @param view 百度云播放器渲染视图。
 @param viaHardwareAccelerate 百度云播放器视频解码方式。
 */
- (void)setProviderBDCloudMediaPlayerView:(UIView *)view
                    viaHardwareAccelerate:(BOOL)viaHardwareAccelerate;

/**
 配置VR渲染交互模式。

 @param interactiveMode 交互模式。
 */
- (void)setInteractiveMode:(BDCVRModeInteractive)interactiveMode;

/**
 配置VR渲染渲染模式。

 @param displayMode 渲染模式。
 */
- (void)setDisplayMode:(BDCVRModeDisplay)displayMode;

/**
 配置VR资源类型

 @param projectionMode VR资源类型。
 */
- (void)setProjectionMode:(BDCVRModeProjection)projectionMode;

/**
 配置是否允许捏合手势。

 @param pinch 是否允许。
 */
- (void)setPinchEnabled:(BOOL)pinch;

/**
 配置VR渲染视图的父视图。

 @param parentView 父视图。
 */
- (void)setRenderOn:(UIView *)parentView;
@end

/**
 内部使用，外部无需调用。
 */
@interface BDCloudVRConfiguration (Private)
/**
 配置BDCloudVRRenderControl必要函数。

 @return 配置BDCloudVRRenderControl必要参数。
 */
- (id)configRender;

/**
 渲染VR图像。

 @param image VR渲染图像。
 */
- (void)provider:(UIImage *)image;
@end
